@@ -511,7 +511,7 @@ DecodeTreeElement *DecodeTree::getNextElement(uint8_t &elemID)
     if(m_decode_elem_iter < 0x80)
     {
         // find a none zero entry or end of range
-        while((m_decode_elements[m_decode_elem_iter] == 0) && (m_decode_elem_iter < 0x80))
+        while((m_decode_elem_iter < 0x80) && (m_decode_elements[m_decode_elem_iter] == 0))
             m_decode_elem_iter++;
 
         // return entry unless end of range
@@ -621,6 +621,7 @@ ocsd_err_t DecodeTree::addPacketPrinter(uint8_t CSID, bool bMonitor, ItemPrinter
             switch (protocol)
             {
             case  OCSD_PROTOCOL_ETMV4I:
+            case  OCSD_PROTOCOL_ETE:
             {
                 PacketPrinter<EtmV4ITrcPacket> *pTPrinter = dynamic_cast<PacketPrinter<EtmV4ITrcPacket> *>(pPrinter);
                 if (bMonitor)
